import unittest
from fs.memoryfs import MemoryFS
from fs.filefs import FileFS
from fs.errors import ResourceNotFound, Unsupported

class TestFileFS(unittest.TestCase):
    def setUp(self):
        # Create an in-memory filesystem and files
        self.memfs = MemoryFS()
        self.memfs.writetext("foo.txt", "foo")
        self.memfs.makedir("foo")
        self.memfs.writetext("foo/bar.txt", "bar")
        self.memfs.writetext("log.txt", "")
        self.filefs = FileFS()
        self.filefs.add_file("foo.txt", self.memfs)
        self.filefs.add_file("foo/bar.txt", self.memfs, name="foobar.txt")
        self.filefs.add_file("log.txt", self.memfs)

    def tearDown(self):
        self.memfs.close()

    def test_listdir(self):
        self.assertListEqual(self.filefs.listdir("/"), ["foo.txt", "foobar.txt", "log.txt"])

    def test_read_file(self):
        with self.filefs.openbin("foo.txt") as f:
            self.assertEqual(f.read(), b"foo")
        with self.filefs.openbin("foobar.txt") as f:
            self.assertEqual(f.read(), b"bar")

    def test_file_not_found(self):
        with self.assertRaises(ResourceNotFound):
            self.filefs.openbin("nofile.txt")

    def test_write_file(self):
        with self.filefs.openbin("log.txt", mode="wb") as f:
            f.write(b"new content")
        with self.memfs.openbin("log.txt") as f:
            self.assertEqual(f.read(), b"new content")

    def test_write_new_file(self):
        with self.assertRaises(ResourceNotFound):
            with self.filefs.openbin("new.txt", mode="wb") as f:
                f.write(b"new content")

    def test_add_dir(self):
        with self.assertRaises(Unsupported):
            self.filefs.add_file("foo", self.memfs)
    

if __name__ == "__main__":
    unittest.main()



from fs.base import FS
from fs.info import Info
from fs.errors import ResourceNotFound, ResourceReadOnly, Unsupported
from fs.path import basename, normpath
from typing import Dict

class FileFS(FS):
    """
    A custom filesystem that allows mounting files from other filesystems.
    """
    _meta = {
        "virtual": True,
        "read_only": False,
        "case_insensitive": False,
    }

    def __init__(self):
        super().__init__()
        self._files: Dict[str, tuple[FS, str]] = {}

    def add_file(self, path: str, fs: FS, name=None):
        """
        Mount a file from another filesystem.
        """
        path = normpath(path)
        if fs.isdir(path):
            raise Unsupported(path)
        self._files[normpath(name or basename(path))] = (fs, path)

    def remove_file(self, name):
        if name in self._files:
            del self._files[normpath(name)]

    def listdir(self, path):
        if path != "/":
            raise ResourceNotFound(path)
        return list(self._files.keys())

    def _delegate(self, path) -> tuple[FS, str]:
        path = normpath(path)
        if path not in self._files:
            raise ResourceNotFound(path)
        return self._files[path]

    def openbin(self, path, mode="r", **kwargs):
        fs, dst_path = self._delegate(path)
        return fs.openbin(dst_path, mode=mode, **kwargs)

    def getinfo(self, path, namespaces=None):
        fs, dst_path = self._delegate(path)
        return fs.getinfo(dst_path, namespaces)

    def setinfo(self, path, info):
        fs, dst_path = self._delegate(path)
        return fs.setinfo(dst_path, info)

    def makedir(self, path, permissions=None, recreate=False):
        raise Unsupported(path)

    def remove(self, path):
        raise Unsupported(path)

    def removedir(self, path):
        raise Unsupported(path)